<?xml version="1.0" encoding="US-ASCII"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:c="http://www.CraneSoftwrights.com/ns/ublsec"
                xmlns:ds="http://www.w3.org/2000/09/xmldsig#"
                version="1.0" exclude-result-prefixes="c">

<!--
Add an external file of object information, recognizing any embedded signals.

$Id: Crane-SigObject.xsl,v 1.4 2010/11/27 21:40:04 admin Exp $

Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-dev.htm

Redistribution and use of components created by Crane Softwrights Ltd. in 
source and binary forms, with or without modification, are permitted 
provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- The name of the author may not be used to endorse or promote products
  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5

THE AUTHOR MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS CODE FOR ANY
PURPOSE. PLEASE CONSULT THE RESPECTIVE RESPONSIBLE PARTIES REGARDING COPYRIGHT
AND LEGAL OBLIGATIONS FOR COMPONENTS NOT WRITTEN BY CRANE SOFTWRIGHTS LTD.
-->

<!--UBL optionally added information-->
<xsl:param name="objectFileName"/> <!--relative URI is relative to UBL input-->
<xsl:variable name="c:objectFile" select="document($objectFileName,/)"/>
<xsl:param name="RefIDs" select="''"/>

<!--add as many references for Id= or xml:id= attributes signaled as signed-->
<xsl:template name="c:SigReferences">
  <!--any references implied by the extension-->
  <xsl:if test="$objectFileName">
    <xsl:if test="not($c:objectFile)">
      <xsl:message terminate="yes">
        <xsl:text>The given parameter objectFileName of "</xsl:text>
        <xsl:value-of select="$objectFileName"/>
        <xsl:text>" does not open as an XML file (a relative URI </xsl:text>
        <xsl:text>is relative to the input to the transformation).</xsl:text>
      </xsl:message>
    </xsl:if>
    <xsl:if test="not($c:objectFile/ds:Object)">
      <xsl:message terminate="yes">
        <xsl:text>The given objectFileName "</xsl:text>
        <xsl:value-of select="$objectFileName"/>
        <xsl:text>" does not have </xsl:text>
        <xsl:text>{http://www.w3.org/2000/09/xmldsig#}Object </xsl:text>
        <xsl:text>as its document element.</xsl:text>
      </xsl:message>
    </xsl:if>
    <xsl:apply-templates select="$c:objectFile//*" mode="c:SigReferences"/>
  </xsl:if>
  <!--any references specified on the command line-->
  <xsl:call-template name="c:AddedReferences"/>
</xsl:template>

<!--walk through the white-space-separated list of added URI references-->
<xsl:template name="c:AddedReferences">
  <xsl:param name="refs" 
             select="concat(normalize-space($RefIDs),' ')"/>
  <xsl:variable name="this" select="substring-before($refs,' ')"/>
  <xsl:if test="$this">
    <!--add this reference first-->
    <xsl:call-template name="c:AddOneReference">
      <xsl:with-param name="ref" select="$this"/>
    </xsl:call-template>
    <!--now add others references-->
    <xsl:call-template name="c:AddedReferences">
      <xsl:with-param name="refs" select="substring-after($refs,' ')"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<!--for each qualifying element (that is, signaled by a c:sign="{attr-name}" 
    attribute) with either Id= or xml:id=, thus create a signing reference-->
<xsl:template match="*[normalize-space(@c:sign)!='']
                      [@*[name(.)=normalize-space(../@c:sign)]]" priority="3"
              mode="c:SigReferences"
              name="c:AddOneReference">
  <!--if a reference isn't passed, then it is based on the current node-->
  <xsl:param name="ref"
             select="concat('#',@*[name(.)=normalize-space(../@c:sign)])"/>
  <!--this is *so* much easier in XSLT 2 than this XSLT 1 stuff!-->
  <xsl:variable name="useDigestMethod">
    <xsl:choose>
      <xsl:when test="@c:digestMethod">
        <xsl:value-of select="@c:digestMethod"/>
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$digestMethod"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <!--compose the reference to be used-->
  <xsl:if test="true()" xml:space="preserve">
    <ds:Reference URI="{$ref}"
                  ><xsl:if test="@c:xpath">
      <ds:Transforms>
        <ds:Transform Algorithm="http://www.w3.org/TR/1999/REC-xpath-19991116">
          <XPath><xsl:value-of select="@c:xpath"/></XPath>
        </ds:Transform>
      </ds:Transforms></xsl:if>
      <ds:DigestMethod Algorithm="{$useDigestMethod}"/>
      <ds:DigestValue/>
    </ds:Reference></xsl:if>
</xsl:template>

<!--c:sign must be empty to signal an element with xml:id= or Id= isn't
    to be signed; if not present this might be an oversight-->
<xsl:template match="*[normalize-space(@c:sign)!='']
                      [not(@*[name(.)=normalize-space(../@c:sign)])]"
              mode="c:SigReferences" priority="2">
  <xsl:message terminate="yes">
    <xsl:text>Element found with inappropriate use of c:sign= </xsl:text>
    <xsl:text>attribute.  No attribute found with the name "</xsl:text>
    <xsl:value-of select="normalize-space(@c:sign)"/>
    <xsl:text>".
</xsl:text>
    <xsl:call-template name="c:ExposeXPath"/>
  </xsl:message>
</xsl:template>

<!--no other signature attributes can be used if c:sign= not properly used-->
<xsl:template match="*[@c:*] | c:*" mode="c:SigReferences">
  <xsl:message terminate="yes">
    <xsl:text>Element found with inappropriate use of Crane </xsl:text>
    <xsl:text>namespace.
</xsl:text>
    <xsl:call-template name="c:ExposeXPath"/>
  </xsl:message>
</xsl:template>

<!--all other elements are acceptable and don't generate a reference-->
<xsl:template match="*" mode="c:SigReferences"/>

<!--========================================================================-->

<!--preserve the referenced file of ds:Object information-->
<xsl:template name="c:SigObject">
  <xsl:if test="$objectFileName">
    <xsl:apply-templates select="$c:objectFile/ds:Object"
                         mode="c:SigObject"/>
    <xsl:text>
</xsl:text>
  </xsl:if>
</xsl:template>

<xsl:variable name="c:ns" select="document('')/*/namespace::c"/>

<!--preserve element except for the Crane security namespace-->
<xsl:template match="*" mode="c:SigObject" priority="1">
  <xsl:element name="{name(.)}" namespace="{namespace-uri(.)}">
    <xsl:copy-of select="namespace::*[.!=$c:ns]"/>
    <xsl:apply-templates select="@*|node()" mode="c:SigObject"/>
  </xsl:element>
</xsl:template>

<xsl:template match="c:*" mode="c:SigObject" priority="2"/>

<xsl:template match="@c:*" mode="c:SigObject" priority="2"/>

<!--identity for all other nodes which must be leaf nodes-->
<xsl:template match="@*|node()" mode="c:SigObject">
  <xsl:copy/>
</xsl:template>

<!--diagnostic exposition of XPath address to current element or attr node-->
<xsl:template name="c:ExposeXPath">
  <xsl:for-each select="ancestor-or-self::*">
    <xsl:text/>/<xsl:value-of select="name(.)"/>
    <xsl:if test="position()>1">[<xsl:number/>]</xsl:if>
  </xsl:for-each>
  <xsl:if test="count(../@*|.)=count(../@*)">
    <xsl:text/>/@<xsl:value-of select="name(.)"/>
  </xsl:if>
</xsl:template>

</xsl:stylesheet>