<?xml version="1.0" encoding="US-ASCII"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:ds="http://www.w3.org/2000/09/xmldsig#"
                version="1.0">

<!--

Construct a script file to be executed to validate all of the signatures
one at a time, because 'xmlsec' only validates one at a time.

$Id: Crane-VerifyUBL.xsl,v 1.3 2010/11/27 21:40:05 admin Exp $

Implementation note:  the non-DOS interface has not been fully developed
                      or tested, just sketched for future use once a non-DOS
                      executable version of 'xmlsec' is found

Copyright (C) - Crane Softwrights Ltd.
              - http://www.CraneSoftwrights.com/links/res-dev.htm

Redistribution and use of components created by Crane Softwrights Ltd. in 
source and binary forms, with or without modification, are permitted 
provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- The name of the author may not be used to endorse or promote products
  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5

THE AUTHOR MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS CODE FOR ANY
PURPOSE. PLEASE CONSULT THE RESPECTIVE RESPONSIBLE PARTIES REGARDING COPYRIGHT
AND LEGAL OBLIGATIONS FOR COMPONENTS NOT WRITTEN BY CRANE SOFTWRIGHTS LTD.
-->

<xsl:output method="text"/>

<xsl:param name="infile"/>

<xsl:param name="isDOS" select="'no'"/>

<xsl:template match="/">
  <xsl:variable name="signatures" select="//ds:Signature"/>

  <!--prep the script variables-->
  <xsl:choose>
    <xsl:when test="$isDOS='yes'">
    <xsl:text>@echo off
set CraneVerifyUBLBad=0
</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>CraneVerifyUBLBad=0
</xsl:text>
    </xsl:otherwise>
  </xsl:choose>

  <!--indicate if no work to be done-->
  <xsl:if test="not($signatures)">
    <xsl:text>echo No signatures found in input file.
</xsl:text>
  </xsl:if>

  <!--invoke the work to be done-->
  <xsl:for-each select="$signatures">
    <xsl:text>echo Checking signature </xsl:text>
    <xsl:value-of select="concat(position(),' of ',last(),'...')"/>
    <xsl:text>
</xsl:text>
    <xsl:text>xmlsec --verify %signIDAttrs% --node-xpath </xsl:text>
    <xsl:text>"(//*[local-name()='Signature' and namespace-uri()=</xsl:text>
    <xsl:text>'http://www.w3.org/2000/09/xmldsig#'])[</xsl:text>
    <xsl:value-of select="position()"/>
    <xsl:text>]" </xsl:text>
    <xsl:value-of select="$infile"/>
    <xsl:text>
</xsl:text>
    <xsl:choose>
      <xsl:when test="$isDOS='yes'">
        <xsl:text>set Crane-VerifyUBL-This=%errorlevel%
set /A CraneVerifyUBLBad=CraneVerifyUBLBad + %Crane-VerifyUBL-This%
if %Crane-VerifyUBL-This% neq 0 echo Signature </xsl:text>
        <xsl:value-of select="position()"/>
        <xsl:text> failed
</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>if [[ "$?" != "0" ]]; then echo Signature </xsl:text>
        <xsl:value-of select="position()"/>
        <xsl:text> failed ; CraneVerifyUBLBad += 1
</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>

  <!--clean up when done-->
  <xsl:choose>
    <xsl:when test="$isDOS='yes'">
      <xsl:text>if %CraneVerifyUBLBad% neq 0 goto :problem
</xsl:text>
      <xsl:choose>
        <xsl:when test="count($signatures)>1"><xsl:text>
echo All </xsl:text><xsl:value-of select="count($signatures)"/>
      <xsl:text> signatures valid
</xsl:text>
        </xsl:when>
        <xsl:when test="count($signatures)=1">
          <xsl:text>echo The one signature found is valid
          </xsl:text>
        </xsl:when>
      </xsl:choose>
      <xsl:text>goto :done
:problem
echo Of </xsl:text><xsl:value-of select="count($signatures)"/>
      <xsl:text> signature</xsl:text>
      <xsl:if test="count($signatures)>1">s</xsl:if>
      <xsl:text> found, %CraneVerifyUBLBad% failed
set CraneVerifyUBLBad=
exit /b 1
:done
exit /b 0
</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>if [[ $CraneVerifyUBLBad != 0 ]]; then 
echo Of </xsl:text><xsl:value-of select="last()"/>
      <xsl:text>signature</xsl:text><xsl:if test="last()>1">s</xsl:if>
      <xsl:text>, $CraneVerifyUBLBad failed
else
echo All </xsl:text><xsl:value-of select="last()"/>
      <xsl:text> signature</xsl:text><xsl:if test="last()>1">s</xsl:if>
      <xsl:text> valid
fi
</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>
